clear maplemex
syms tau q q1 q2 q3 p1 p2 p3 real
%lambda = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Symbolic manipulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Hper = Hp([q1 q2 q3], [p1 p2 p3],...
    epsilon1, epsilon2, epsilon3, epsilon4, epsilon5, Hp1, Hp2, Hp3, Hp4, Hp5);
dHp_q1 = diff(Hper,q1);
dHp_q2 = diff(Hper,q2);
dHp_q3 = diff(Hper,q3);
dHp_p1 = diff(Hper,p1);
dHp_p2 = diff(Hper,p2);
dHp_p3 = diff(Hper,p3);
if dHp_p1 ~= 0 | dHp_p2 ~= 0 | dHp_p3 ~= 0
    errordlg('With a 2nd order ODE solver the perturbation Hamiltonian MUST be function of the position vector only','Velocity not allowed')
    return
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       Write to file and integrate the differential equations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen( 'StandardMethod/Stand2ndOrdDiffEq.m', 'wt');
fprintf(fid, 'function dotdotW = Stand2ndOrdDiffEq(tau,W)\n');
if strcmp(get(handles.waitbars, 'Checked'),'on')
    fprintf(fid, 'global pick tau_total\n');              %%%%%%%%      waitbar
    fprintf(fid, 'if tau > pick + tau_total/100;\n');     %%%%%%%%      waitbar
    fprintf(fid, '    waitbar(tau/tau_total);\n');        %%%%%%%%      waitbar
    fprintf(fid, '    pick = tau;\n');                    %%%%%%%%      waitbar
    fprintf(fid, 'end\n');                                %%%%%%%%      waitbar
end
fprintf(fid, 'dotW = zeros(6,1);\n');
fprintf(fid, 'dotdotW = zeros(3,1);\n');
%fprintf(fid, 'global lambda;\n');
fprintf(fid, '%%\n%% SUBSTITUTIONS\n%%\n');
fprintf(fid, ' q1 = W(1);\n q2 = W(2);\n q3 = W(3);\n');
fprintf(fid, ' q = sqrt(q1^2+q2^2+q3^2);\n');
fprintf(fid, '%%\n%% EQUATIONS\n%%\n');
fprintf(fid, ' dotdotW(1) = %s;\n dotdotW(2) = %s;\n dotdotW(3) = %s;\n',...
    char(-q1/q^3-dHp_q1), char(-q2/q^3-dHp_q2), char(-q3/q^3-dHp_q3));
fclose(fid);

if strcmp(get(handles.waitbars, 'Checked'),'on')
   global pick                                              %%%%%     waitbar
   h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
   pick = -tau_total/100;                                   %%%%%     waitbar
end

StepSize2 = str2num(get(handles.edit_StepSize2,'String'));
StepSize2 = 2*pi*sm_ax_o^(3/2)*StepSize2;
options = gniset('StepSize',StepSize2);

tic
[t_c,W,dotW] = eval([ODE2, '(''Stand2ndOrdDiffEq'', [0 2*pi*sm_ax_o^(3/2)*tau_total], [q_vec_o''; p_vec_o''], options)']);
t_elaps = toc;
t_elaps = num2str(t_elaps); 
if strcmp(get(handles.waitbars, 'Checked'),'on')
   close(h)                                                 %%%%%     waitbar
end
  
if strcmp(get(handles.perform, 'Checked'),'on')
    if strcmp(num2str(perfid), '')
        perfid = fopen( 'Perform/Perform.txt', 'wt');
        fprintf(perfid, '*************************************************************************************************\n');
    end
    fprintf(perfid, '                               Standard method          \n\n');
    fprintf(perfid, ['ODE solver ',ODE2,', Cycle number ',num2str(tau_total),...
             ', Step size ',num2str(StepSize2/(2*pi*sm_ax_o^(3/2))),...
             ', Integration time ',t_elaps,' sec.\n']);
    fprintf(perfid, '*************************************************************************************************\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Calculate Trajectories
step_total = size(W,1);
step_i = floor(tau_i/tau_total*step_total)+1;
step_f = floor(tau_f/tau_total*step_total);
t = 1/(2*pi)*t_c(step_i:step_f);

W(:,4) = dotW(:,1);
W(:,5) = dotW(:,2);
W(:,6) = dotW(:,3);

q_vec = W(step_i:step_f,1:3);
p_vec = W(step_i:step_f,4:6);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%